@@ -1,4 +1,5 @@
-knots <- function(Fn, ...)UseMethod('knots')
+## knots <- function(Fn, ...)UseMethod('knots')
+#.knots_default <- function(Fn, ...)UseMethod('knots')
 
 knots.basisfd <- function(Fn, interior = TRUE, ...) {
 ##
